#include "includes.h"



INT8U gu8SCIData[128];
INT8U gu8SCIDataFlag;

extern INT8S gi8AppStatus;


/*-------------------------------------------------------------------------------------------------------------
  INTERRUPT HANDLER
-------------------------------------------------------------------------------------------------------------*/
void interrupt Vscirx(void)
{
  INT8U u8status, u8dummy;
  /* ensure Rx data buffer is full */
  do
  {
    u8status = SCIS1;
  } while( (u8status & 0x20) == 0 );

  /* Check for Errors (Framing, Noise, Parity) */
  if( (u8status & 0x07) != 0 )
  {
    u8dummy = SCID;
    return;
  }

  /* Good Data. */
  gu8SCIData[0] = SCID; /* load SCI register to data */
  gu8SCIData[1] = 0;
  gu8SCIDataFlag = 1;
}



/************************************************************
Function: SCIInitGeneric
arg types (u16, u16, u8)

Function initializing the SCI to the requested baud rate
based on the supplied busclock in kilohertz on the com port
that is selected

************************************************************/



void UART_Init(INT32U u32BusClock, INT32U u32BaudRate, INT8U u8ComNumber)
{

  INT16U u16Sbr = 0;
  u16Sbr = (INT16U) ((u32BusClock) / (u32BaudRate * 16));

  switch( u8ComNumber )
  {
  case 1:
    SCI1BDL = (INT8U) u16Sbr;
    u16Sbr = u16Sbr >> 8;
    SCI1BDH = (INT8U) u16Sbr;
    SCI1C2 = INIT_SCI2C2;
    break;

  case 2:
    SCI2BDL = (INT8U) u16Sbr;
    u16Sbr = u16Sbr >> 8;
    SCI2BDH = (INT8U) u16Sbr;
    SCI2C2 = INIT_SCI2C2;
    break;

  default:
    SCI1BDL = (INT8U) u16Sbr;
    u16Sbr = u16Sbr >> 8;
    SCI1BDH = (INT8U) u16Sbr;
    SCI1C2 = INIT_SCI2C2;
    break;
  }
}


void UART_send_block(INT8U *pStr, INT8U length)
{
  INT8U i;
  for( i=0; i<length; i++ )
  {
    UART_send_byte(pStr[i]);
  }
}

void UART_send_byte(INT8U cData)
{
  while( !SCIS1_TDRE );  /* ensure Tx data buffer empty */
  SCID = cData; /* load data to SCI2 register */
  while( !SCIS1_TC ); /* wait for Tx complete */
}

void UART_send_string(INT8U *pStr)
{
  INT16U i;
  INT16U nStrLen=strlen(pStr);

  for( i=0; i<nStrLen; i++ )
  {
    UART_send_byte(pStr[i]);
  }
}

INT8U UART_accept_cmd(char *pPrompt, char *pBuffer, INT8U u8Size)
{


  INT8U u8ExitFlag = FALSE;
  INT8U u8Offset=0;

  //Echo the string for the command line
  UART_send_string(pPrompt);

  //Keep processing SCI inputs until Size is reached or terminating NULL
  //occurs.
  while( u8ExitFlag == FALSE )
  {
    if( gu8SCIDataFlag == 1 )
    {
      gu8SCIDataFlag = 0;
      //Check the terminating string
      switch( gu8SCIData[0] )
      {
      //Check for Terminating string
      case 0x0D:
        u8ExitFlag = TRUE;
        pBuffer[u8Offset] = 0; //Terminate sting with null.
        break;

      case '\b':
        if( u8Offset != 0 )
          pBuffer[u8Offset--] = 0;
        else
          pBuffer[u8Offset] = 0;
        break;

      case '\n':
      default:
        pBuffer[u8Offset++] = gu8SCIData[0];
        break;
      }
      //Check the buffersize not to overrun buffer.
      if( u8Offset == u8Size )
      {
        u8ExitFlag = TRUE;
      }
    }
  }

  return u8Offset;

}

void UART_send_block_as_hex_str(char *pStr, INT8U length)
{
  int i;
  INT8U low_nibble=0, hi_nibble=0;
  INT8U fourbit=0;

  for( i=0; i<length; i++ )
  {

    low_nibble = pStr[i] & 0x0F;
    hi_nibble = (pStr[i] & 0xF0) >> 4;

    if( hi_nibble <= 9 )
    {
      UART_send_byte(hi_nibble + 0x30);
    }
    else
    {
      UART_send_byte(hi_nibble + 0x37);
    }

    if( low_nibble <= 9 )
    {
      UART_send_byte(low_nibble + 0x30);
    }
    else
    {
      UART_send_byte(low_nibble + 0x37);
    }
  }
}
