#include "includes.h"

#define  CLKO_4MHZ  2 //   CLKO = 4 MHz

/* Global Variables */
byte          g_phy_mode; /* needed for s-mac, application can read this variable */
INT8U         channel = 0;
extern INT32U cnt_0_8_ms;
INT8U         energy;
INT8U         max_energy = 0xFF;

#define       BUF_LEN 128
TRxPacket     rxpack_struct;
INT8U         data_buf[BUF_LEN];
INT32U        pack_cnt;
INT32U        lost_pack_cnt;
INT32U        cur_pack_nn;
char          str[17];


typedef struct
{
  const char*    name;
  void           (*func)(void);
  void           (*initfunc)(void);
} TMode;
static void InitEnergyLevSampler(void);
static void InitPacketStreamMonitor(void);
static void EnergyLevSampler(void);
static void PacketStreamMonitor(void);

const TMode modes[2]=
{
  {"En.Lev.Sampler",EnergyLevSampler,InitEnergyLevSampler},
  {"Pk.Stream.Mon.",PacketStreamMonitor,InitPacketStreamMonitor},
};
#define  MODESNUM (sizeof(modes)/sizeof(modes[0]))
char          mode;


/*-------------------------------------------------------------------------------------------------------------
    
-------------------------------------------------------------------------------------------------------------*/
void main(void)
{
  SOPT = 0x73;                   /* Turn off the watchdog. */
  IOports_Init();

  Timer1_init();
  Timer1_delay(T_TO_T1CYCLE(0.040)); //   ~200 

  SPI_Init();

  Phy_Init(CLKO_4MHZ);               //   CLKO = 4 MHz
  Phy_Set_Channel(channel);          // Set channel to 0 (2.405GHz)
  Phy_Adjust_Power_Level(MAX_POWER); // Set MAX power setting
  //  Phy_Set_int_mode();                //    Phy  

  Phy_Set_poll_mode();               //   Phy    

  ICG_Init();                        //    40 MHz

  UART_Init(BUS_FREQ,57600, DEFAULT_SCI_PORT);

  LCD_Init (2,16);
  LCD_ClrScr();
  LCD_DispStr (0, 0, "Init done!");

  ENABLE_INTERRUPT;            /* Turn on system interrupts */

  DELAY_MS(1000);
  sprintf(str,"ChipID=%X",Phy_Read_RfICVersion());
  LCD_DispStr (1, 0, str);
  DELAY_MS(1000);
  LCD_ClrLine(1);
  LCD_ClrLine(0);
  LCD_DispStr (0, 0, "Recieving test");


  mode = 0;
  modes[mode].initfunc();
  LCD_ClrLine(0);
  sprintf(str,"%s",modes[mode].name);
  LCD_DispStr (0, 0, str);
  DELAY_MS(1000);

  for( ;; )
  {

    modes[mode].func();

    if( Get_SW_state(SW1)== TRUE ) //   
    {
      if( channel == 15 )  channel = 0;
      else                    channel += 1;
      Phy_Set_Channel(channel);
      max_energy = 0xFF;
      LCD_ClrLine(1);
      sprintf(str,"Ch=%d",channel);
      LCD_DispStr (1, 0, str);
      DELAY_MS(500);
    }
    if( Get_SW_state(SW4)== TRUE )
    {
      mode++;
      if( mode>=MODESNUM ) mode =0;
      modes[mode].initfunc();
      LCD_ClrScr();
      sprintf(str,"%s",modes[mode].name);
      LCD_DispStr (0, 0, str);
      DELAY_MS(1000);
    }
    if( Get_SW_state(SW3)== TRUE )
    {
      cur_pack_nn = 0;
    }


  }
}

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
static void EnergyLevSampler(void)
{
  INT8U status;
  static INT8U ledst;

  if( Phy_poll_interrupt() == TRUE )
  {
    status = SPI_Read(IRQ_STATUS);
    if( status & CCA_IRQ )
    {
      ledst = ledst ^1;
      LED1 = ledst;
      status = ((SPI_Read(RX_STATUS) & 0xFF00) >> 8)/2;
      pack_cnt++;
      while( !SCIS1_TDRE );
      SCID =  status;
      while( !SCIS1_TDRE );
      SCID =  channel | 0x80;
    }
    Phy_Set_Mode(CCA_MODE);
  }
}

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
static void PacketStreamMonitor(void)
{
  INT8U  status;
  INT32U nn;
  static INT8U ledst;

  if( Phy_poll_interrupt() == TRUE )
  {
    status = SPI_Read(IRQ_STATUS);
    if( status & RX_RCVD_IRQ__RX_STRM_IRQ )
    {
      if( Phy_receive_packet(status)==TRUE )
      {
        ledst = ledst ^1;
        LED2 = ledst;
        //   4-         
        //     
        nn  = (INT32U)rxpack_struct.pdata[0] << 24;
        nn |= (INT32U)rxpack_struct.pdata[1] << 16;
        nn |= (INT32U)rxpack_struct.pdata[2] << 8;
        nn |= (INT32U)rxpack_struct.pdata[3];

        if( cur_pack_nn == 0 )
        {
          cur_pack_nn = nn + 1;
          lost_pack_cnt = 0;
        }
        else
        {
          if( nn<cur_pack_nn )
          {
            lost_pack_cnt = 0;
            cur_pack_nn = nn + 1;
          }
          else
          {
            lost_pack_cnt += (nn - cur_pack_nn);
            cur_pack_nn = nn + 1;
          }
        }
        pack_cnt++;
        LCD_ClrScr();
        sprintf(str,"Lost=%ld",lost_pack_cnt);
        LCD_DispStr (0, 0, str);
        sprintf(str,"N=%ld LQ=%d",pack_cnt,rxpack_struct.lq/2);
        LCD_DispStr (1, 0, str);
      }
    }
  }
}

static void InitEnergyLevSampler(void)
{
  Phy_Set_Mode(CCA_MODE);
  pack_cnt = 0;
  LED1 = 1;
  LED2 = 1;
  LED3 = 1;
}
static void InitPacketStreamMonitor(void)
{
  rxpack_struct.max_dlen = BUF_LEN;
  rxpack_struct.pdata    = data_buf;
  Phy_Init_Reciving(&rxpack_struct);
  pack_cnt = 0;
  lost_pack_cnt = 0;
  cur_pack_nn = 0;
  LED1 = 1;
  LED2 = 1;
  LED3 = 1;
}

/*-------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------*/
static void Test_whetstone(void)
{
  INT32U loop;

  LCD_ClrScr();
  LCD_DispStr (0, 0, "Whetstone");

  loop = 0;
  while( 1 )
  {
    INT16U cnt;
    float f;
    cnt_0_8_ms = 0;
    Whetstone();
    cnt = cnt_0_8_ms;
    f = cnt*0.0008;
    loop++;
    sprintf(str,"%f %ld",f, loop);
    LCD_ClrLine(1);
    LCD_DispStr (1, 0, str);
  }
}


