#define BUS_FREQ              20000000u
#define CPU_FREQ              40000000u

#define DB9_SCI_PORT          1

#define DEFAULT_SCI_PORT      2

/**************************************************************
Define the SCI perameters
**************************************************************/

#if (DEFAULT_SCI_PORT == 1)
  #define  SCIBDH     SCI1BDH
  #define  SCIBDL     SCI1BDL
  #define  SCIC1      SCI1C1
  #define  SCIC2      SCI1C2
  #define  SCIS1      SCI1S1
  #define  SCIS2      SCI1S2
  #define  SCIC3      SCI1C3
  #define  SCID       SCI1D

  #define  SCIS1_TDRE  SCI1S1_TDRE
  #define  SCIS1_TC    SCI1S1_TC

#else

  #define  SCIBDH     SCI2BDH
  #define  SCIBDL     SCI2BDL
  #define  SCIC1      SCI2C1
  #define  SCIC2      SCI2C2
  #define  SCIS1      SCI2S1
  #define  SCIS2      SCI2S2
  #define  SCIC3      SCI2C3
  #define  SCID       SCI2D

  #define  SCIS1_TDRE  SCI2S1_TDRE
  #define  SCIS1_TC    SCI2S1_TC

#endif
#define  SCI_DEFAULT_BAUD  38400

/**************************************************************
Define the LED perameters
**************************************************************/

#define LED1                    PTDD_PTDD4
#define LED1DIR                 PTDDD_PTDDD4
#define LED2                    PTDD_PTDD5
#define LED2DIR                 PTDDD_PTDDD5
#define LED3                    PTDD_PTDD6
#define LED3DIR                 PTDDD_PTDDD6
#define LED4                    PTDD_PTDD7
#define LED4DIR                 PTDDD_PTDDD7
#define LED5                    PTCD_PTCD4
#define LED5DIR                 PTCDD_PTCDD4

#define LED_ON                  0
#define LED_OFF                 1
#define DDIR_OUTPUT             1
#define DDIR_INPUT              0

/***********************************************
Define Buzzer
***********************************************/


#define BUZZER         PTDD_PTDD2
#define BUZZERDIR      PTDDD_PTDDD2
#define BUZZER_ON      1
#define BUZZER_OFF     0

#define BUZZER_INIT_MACRO BUZZER      = BUZZER_OFF; \
                          BUZZERDIR   = DDIR_OUTPUT;

/***********************************************
Define Switches
***********************************************/


/* SW1 on schematic */
#define PB0 PTAD_PTAD2
#define PB0PU PTAPE_PTAPE2
#define PB0DIR PTADD_PTADD2
#define PB0IE KBI1PE_KBI1PE2

/* SW2 on schematic */
#define PB1 PTAD_PTAD3
#define PB1PU PTAPE_PTAPE3
#define PB1DIR PTADD_PTADD3
#define PB1IE KBI1PE_KBI1PE3

/* SW3 on schematic */
#define PB2 PTAD_PTAD4
#define PB2PU PTAPE_PTAPE4
#define PB2DIR PTADD_PTADD4
#define PB2IE KBI1PE_KBI1PE4

/* SW4 on schematic */
#define PB3 PTAD_PTAD5
#define PB3PU PTAPE_PTAPE5
#define PB3DIR PTADD_PTADD5
#define PB3IE KBI1PE_KBI1PE5

/* Define the KBI Pins */
#define KBI_SW1 0x04
#define KBI_SW2 0x08
#define KBI_SW3 0x10
#define KBI_SW4 0x20



#define ENTER_LOW_POWER _asm stop


#define CHANNEL_CHANGE_EVENT 0x01
#define TIMER_EVENT0 0x02
#define TIMER_EVENT1 0x04
#define TIMER_EVENT2 0x08


#define KBI2_EVENT 0x10
#define KBI3_EVENT 0x20
#define KBI4_EVENT 0x40
#define KBI5_EVENT 0x80


#define DELAY_COUNT         0xB000

/* Power Level defines */
#define SPEC_POWER  88    //LEDS 2-4 OFF
#define L1_POWER    80    //LED 2 ON
#define L2_POWER    60    //LED 2,3 ON
#define L3_POWER    40    //LED 2,3,4 ON

/* ARD or EVB low power STOP3*/
#define KBDWAIT()         _asm stop

#define MC13192_CLKO_FREQ    0x01 /* Sets the MC13192 CLKO frequency */
#define LOW_POWER_WHILE() _asm wait
#define ENTER_LOW_POWER   _asm stop


#define INITIAL_STATE      0
#define IDLE_STATE         1
#define WAITING_FOR_ACK    2
#define RECEIVER_ALWAYS_ON 3
#define TRANSMIT_ACK       4
#define TRANSMIT_DATA      5
#define TIMEOUT_STATE      6
#define HIBERNATE_STATE    7
#define OVERFLOW_ERROR     8
#define PRINT_PACKET       9
#define TRANSMIT_MODE      10
#define QUIT               11
#define REGISTER_PRINT     12
#define TMR_PRESCALE       13
#define PRINT_SCI_1        14
#define DOZE_REQUEST_1     15
#define PRINT_SCI_2        16
#define PRINT_SCI_3        17
#define DOZE_REQUEST_2     18
#define DOZE_REQUEST_3     19
#define RX_STATE           20
#define TX_STATE           21
#define RESET_STATE        22
#define SET_LEDS           23
#define GOOD_PACKET        24
#define ECHO_STATE         25
#define FINAL_STATE        26

#define MCU_LOW_POWER_WHILE() _asm wait

//#define IRQFLAG                IRQSC_IRQF
//#define IRQACK()               IRQSC |= 0x04
//#define IRQInit()              IRQSC = 0x14
#define IRQPinEnable()         IRQSC = 0x16

typedef void(*tIsrFunc)(void);

void RxAccel(void);
void TxAccel(void);
void ASCIItoString(INT32U , char *);





