/**
 * Copyright (c) 2004, Freescale Semiconductor
 * Freescale Confidential Proprietary
 *
 * File name :  MC13192_regs.h
 * Project name: SMAC (Simple Media Access Controller)
 *
 * Department : Freescale Radio Products Division
 *
 * Description : This is the SMAC C source driver header file for the MC13192
 * transceiver register map.
 *
 * $Author: flr005 $
 * $Date: 2006/02/23 16:24:36 $
 * $Name:  $
 */

#ifndef _MC13192_REGS_H
#define _MC13192_REGS_H


/* MC13192 soft reset */
#define RESET                   0x00

/* Packet RAM */
#define RX_PKT                  0x01    /* RX Packet RAM */
#define RX_PKT_LEN              0x2D    /* RX Packet RAM Length [6:0] */
#define TX_PKT_RAM              0x02    /* TX Packet RAM */
#define TX_PKT_LEN              0x03    /* TX Packet RAM Length */
#define TX_PKT_LEN_MASK         0x007F  /* TX Packet RAM Length Mask */

/*-------------------------------------------------------------------------
                    Register 05 Description

The attention interrupt mask bit controls the attn_irq interrupt on the IRQ pin. Default is for the
interrupt to be enabled out of reset.
1 = Allows attn_irq to generate an interrupt on the IRQ pin.
0 = When attn_irq status bit is set, IRQ pin is not asserted.*/
#define ATTN_MASK        BIT(15)
/*
The Packet RAM address error interrupt mask bit controls the ram_addr_err interrupt on the IRQ pin.
1 = Allows ram_addr_err to generate an interrupt on the IRQ.
0 = When ram_addr_err status bit is set, IRQ pin is not asserted.*/
#define RAM_ADDR_MASK    BIT(12)
/*
The Packet RAM arbiter busy error interrupt mask bit controls the arb_busy_err interrupt on the IRQ pin.
1 = Allows arb_busy_err to generate an interrupt on the IRQ pin.
0 = When arb_busy_err status bit is set, IRQ pin is not asserted.*/
#define ARB_BUSY_MASK    BIT(11)
/*
The Stream Mode data error interrupt mask bit controls the strm_data_irq interrupt on the IRQ pin.
1 = Allows strm_data_err to generate an interrupt on the IRQ pin.
0 = When strm_data_err status bit is set, IRQ pin is not asserted.*/
#define STRM_DATA_MASK   BIT(10)
/*
The LO1 unlock detect mask bit controls the pll_lock_irq interrupt on the IRQ pin.
1 = Allows pll_lock_irq to generate an interrupt on the IRQ pin.
0 = When pll_lock_irq status bit is set, IRQ pin is not asserted.*/
#define PLL_LOCK_MASK    BIT(9)
/*
The Acoma Mode enable bit controls Doze Mode. Acoma is an enhanced power save mode within Doze.
1 = The MC1321x stays in Doze until ATTN asserted. Event Timer and Prescaler clocks disabled for additional current savings.
0 = Normal operation. Doze is exited by TC2 match or ATTN assertion.*/
#define ACOMA_EN         BIT(8)
/*
The Doze timer interrupt mask bit controls the doze_irq interrupt on the IRQ pin.
1 = Allows doze_irq to generate an interrupt on the IRQ pin.
0 = When doze_irq status bit is set, IRQ pin is not asserted.*/
#define DOZE_MASK        BIT(4)
/*
The Event Timer four interrupt mask bit controls the tmr4_irq interrupt on the IRQ pin.
1 = Allows tmr4_irq to generate an interrupt on the IRQ pin.
0 = When tmr4_irq status bit is set, IRQ pin is not asserted.*/
#define TMR4_MASK        BIT(3)
/*
The Event Timer three interrupt mask bit controls the tmr3_irq interrupt on the IRQ pin.
1 = Allows tmr3_irq to generate an interrupt on the IRQ pin.
0 = When tmr3_irq status bit is set, IRQ pin is not asserted.*/
#define TMR3_MASK        BIT(2)
/*
The Event Timer two interrupt mask bit controls the tmr2_irq interrupt on the IRQ pin.
1 = Allows tmr2_irq to generate an interrupt on the IRQ pin.
0 = When tmr2_irq status bit is set, IRQ pin is not asserted.*/
#define TMR2_MASK        BIT(1)
/*
The Event Timer one interrupt mask bit controls the tmr1_irq interrupt on the IRQ pin.
1 = Allows tmr1_irq to generate an interrupt on the IRQ pin.
0 = When tmr1_irq status bit is set, IRQ pin is not asserted.*/
#define TMR1_MASK        BIT(0)
#define IRQ_MASK         0x05

/*
The reset indicator bit shows whether Register 25 was read since the last RST assertion or program reset.
1 = Register 25 has been read since the last RST or Program Reset event.
0 = Register 25 has not been read since the last RST or Program Reset event.
Note: Reading Register 25 will set Bit 7.
*/
#define RESET_IND         BIT(7)
#define RST_IND           0x25


#define TIMER1_IRQMASK_BIT      0x0001

/*-------------------------------------------------------------------------
                    Register 06 Description

The Transmit Stream Mode bit enables Transmit Streaming Data Transfer Mode
1 = Real-time streaming of TX data to transceiver via SPI bus and IRQ on word-by-word basis.
0 = TX Packet Mode with data preloaded in TX RAM
*/
#define TX_STRM       BIT(12)
/*
The receive Stream Mode bit enables Receive Streaming Data Transfer Mode
1 = Real-time streaming of RX data from transceiver via SPI bus and IRQ on word-by-word basis.
0 = RX Packet Mode with data preloaded in RX RAM*/
#define RX_STRM       BIT(11)
/*
The CCA interrupt mask bit controls the cca_irq interrupt on the IRQ pin.
1 = Allows the cca_irq to generate an interrupt on the IRQ pin.
0 = When icca_irq status bit is set, IRQ pin is not asserted.*/
#define CCA_MASK      BIT(10)
/*
The transmit sent mask bit controls the tx_sent_irq interrupt on the IRQ pin.
1 = Allows the tx_sent_irq to generate an interrupt on the IRQ pin.
0 = When tx_sent_irq status bit is set, IRQ pin is not asserted.*/
#define TX_SENT_MASK  BIT(9)
/*
The packet received interrupt mask bit controls the rx_rcvd_irq interrupt on the IRQ pin.
1 = Allows rx_rcvd_irq to generate an interrupt on the IRQ pin.
0 = When rx_rcvd_irq status bit is set, IRQ pin is not asserted.*/
#define RX_RCVD_MASK  BIT(8)
/*
The timer trigger enable bit determines whether transceiver operation is initiated via Timer Comparator 2 or manually.
1 = The selected transceiver operation is initiated via Timer Comparator 2, or TC2_Prime.
0 = The selected transceiver operation is initiated manually via the RXTXEN pin and by SPI programming.*/
#define TMR_TRIG_EN   BIT(7)
/*
The clear channel assessment type bits selects one of two possible CCA functions.
Algorithm results are reported in field cca_final[7:0], RX_Status Register 2D, Bits 15 - 8.
01 = clear channel assessment
10 = energy detection*/
#define CCA_TYPE1     BIT(5)
#define CCA_TYPE0     BIT(4)
/*
The transceiver operation bits select one of four possible transceiver modes.
00 = Idle (default)
01 = CCA /energy detection
10 = Packet Mode RX
11 = Packet Mode TX*/
#define XCVR_SEQ1     BIT(1)
#define XCVR_SEQ0     BIT(0)
#define CONTROL_A               0x06

/*-------------------------------------------------------------------------
                    Register 07 Description

The load Event Timer bit, when programmed from low to
high, causes the value of the SPI field tmr_cmp1[23:0] to be loaded into the Event Timer.
Write from 0 to 1 to affect load. Rewrite to 0, before writing another 1 to affect another load.
*/
#define TMR_LOAD       BIT(15)
/*
The CT_Bias enable bit controls when the signal CT_Bias is active.
1 = CT_Bias enabled. Output state is defined by Table 3-5.
0 = CT_Bias disabled. Output state is tristated.
*/
#define CT_BIAS_EN     BIT(14)
/*
The CT_Bias Invert bit inverts the sense of the CT_Bias output signal when the CT_Bias is used as a control signal
for dual port operation
The output state of CT_Bias under varying conditions is defined in Table 3-5.
1 = CT_Bias inverted.
0 = CT_Bias not inverted
*/
#define CT_BIAS_INV    BIT(13)
/*
This bit selects the mode of operation of the RF interface, i.e., between Dual Port Mode and Single Port Mode.
1= Single Port Mode selected where RF switch is active and RFIN_M and RFIN_P and bidirectional signals.
0 = Dual Port Mode selected where RFIN_M and RFIN_P are inputs only and PAO_P and PAO_N are separate outputs.
(This is default operation).
*/
#define RF_SWITCH_MODE BIT(12)
/*
The MISO high impedance enable bit either tristates or drives the MISO pin to a logic low when CE is negated.
1 = MISO pin is tristated when CE is negated.
0 = MISO pin is driven to a logic low when CE is negated.
*/
#define MISO_HIZ_EN    BIT(11)
/*
The CLKO enable in Doze Mode bit controls toggling of the CLKO pin during Doze Mode.
1 = The CLKO pin continues to toggle at selected rate during Doze Mode if CLKO is enabled (clko_en = 1).
0 = The CLKO pin stops toggling 128 xtal or reference cycles after the doze_en bit is programmed to 1.
Note: In Doze Mode only, CLKO frequencies of 1.0 MHz or less are available.
*/
#define CLKO_DOZE_EN   BIT(9)
/*
The Stream Mode transmit done interrupt mask bit controls the tx_done_irq interrupt.
1 = Allows the tx_done_irq to generate an interrupt on the IRQ pin.
0 = The tx_done_irq status bit is set, but IRQ pin is not asserted.
Note: tx_done_irq replaces the ram_addr_err when SPI bit use_strm_mode, Register 7, Bit 5 =1.
*/
#define TX_DONE_MASK   BIT(7)
/*
The Stream Mode receive done interrupt mask bit controls the rx_done_irq interrupt.
1 = Allows the rx_done_irq to generate an interrupt on the IRQ pin.
0 = The rx_done_irq status bit is set, but IRQ pin is not asserted.
Note: The rx_done_irq replaces the arb_busy_err when SPI bit use_strm_mode, Register 7, Bit 5 =1.
*/
#define RX_DONE_MASK   BIT(6)
/*
The Stream Mode select bit selects Stream Mode or Packet Mode for transmit and receive data handling.
In Packet Mode, data is stored in RAM and handled as a packet. In Stream Mode, data is managed word-by-word through the SPI.
0 = Packet Mode
1 = Stream Mode
*/
#define USE_STRM_MODE  BIT(5)
/*
The hibernate enable bit can set the MC1321x into its lowest power saving mode without a running time base.
1 = Places the MC1321x into its lowest power operating mode without a running time base.
0 = Normal operation.
*/
#define HIB_EN         BIT(1)
/*
The doze enable bit can set the MC1321x into its lowest power saving mode with a running time base.
1 = Places the MC1321x into its lowest power operating mode with a running time base.
0 = Normal operation.
*/
#define DOZE_EN        BIT(0)
#define CONTROL_B               0x07

#define LO1_COURSE_TUNE         0x8000

/* Main Timer */
#define TIMER_PRESCALE          0x09
#define TIMESTAMP_HI_ADDR       0x26
#define TIMESTAMP_LO_ADDR       0x27
#define TIMESTAMP_HI_MASK       0x00FF

/* Frequency */
#define XTAL_ADJ_ADDR           0x0A
#define CLKS_ADDR               0x0A
#define LO1_IDIV_ADDR           0x0F
#define LO1_NUM_ADDR            0x10
#define PRESCALE_ADDR           0x09

/*
This bit disables the Event Timer Comparator.
1 = Disables the Event Timer Compare function.
0 = Enables the Event Timer Compare function (default).*/
#define TMR_CMP_DIS             BIT(15)
/* Timer comparators */
#define TMR_CMP1_A              0x1B
#define TMR_CMP1_B              0x1C
#define TMR_CMP2_A              0x1D
#define TMR_CMP2_B              0x1E
#define TMR_CMP3_A              0x1F
#define TMR_CMP3_B              0x20
#define TMR_CMP4_A              0x21
#define TMR_CMP4_B              0x22


/* CCA */
/*
Bits 15-8 cca_vt[7:0] - Threshold value for Clear Channel Assessment in dB-linear format, Default is 0x00.
Bits 7-0 power_comp[7:0] - This is a binary value that is added to the measured value of the
CCA operation. The result is stored in cca_final[7:0], Default is 0x8D
*/
#define CCA_THRESHOLD          0x04
#define RX_STATUS              0x2D
#define CCA_THRESH             0x04

/* TX */
#define PA_LVL          0x12

/* GPIO */
/*-------------------------------------------------------------------------
                    Register 0B Description

These bits select output drive strength for GPIO1 through GPIO4.
00 (default) = lowest drive strength;
11 = highest drive strength.
*/
#define GPIO1234_DRV1  BIT(15)
#define GPIO1234_DRV0  BIT(14)
/*
This bit configures GPIO7-GPIO1 as an output.
1 = enabled as output.
0 = disabled as output.
*/
#define GPIO7_OEN      BIT(13)
#define GPIO6_OEN      BIT(12)
#define GPIO5_OEN      BIT(11)
#define GPIO4_OEN      BIT(10)
#define GPIO3_OEN      BIT(9)
#define GPIO2_OEN      BIT(8)
#define GPIO1_OEN      BIT(7)
/*
This bit configures GPIO7-GPIO1 as an input.
1 = enabled as input.
0 = disabled as input.
*/
#define GPIO7_IEN      BIT(6)
#define GPIO6_IEN      BIT(5)
#define GPIO5_IEN      BIT(4)
#define GPIO4_IEN      BIT(3)
#define GPIO3_IEN      BIT(2)
#define GPIO2_IEN      BIT(1)
#define GPIO1_IEN      BIT(0)
#define GPIO_DIR                0x0B

/*-------------------------------------------------------------------------
                    Register 0C Description


These bits select output drive strength for GPIO5 through GPIO7.
00 (default) = lowest drive strength, 11 = highest drive strength.
*/
#define  GPIO567_DRV1 BIT(15)
#define  GPIO567_DRV0 BIT(14)
/*
These bits select output drive strength for signal MISO.
00 (default) = lowest drive strength, 11 = highest drive strength.
*/
#define  MISO_DRV1    BIT(13)
#define  MISO_DRV0    BIT(12)
/*
These bits select output drive strength for signal CLKO.
00 (default) = lowest drive strength, 11 = highest drive strength.
*/
#define  CLKO_DRV1    BIT(11)
#define  CLKO_DRV0    BIT(10)
/*
These bits select output drive strength for signal IRQ.
00 (default) = lowest drive strength, 11 = highest drive strength.
*/
#define  IRQB_DRV1    BIT(9)
#define  IRQB_DRV0    BIT(8)
/*
IRQ pullup enable.
*/
#define  IRQB_PUP_EN  BIT(7)
/*
GPIO7=GPIO1 output value.
*/
#define  GPIO7_O      BIT(6)
#define  GPIO6_O      BIT(5)
#define  GPIO5_O      BIT(4)
#define  GPIO4_O      BIT(3)
#define  GPIO3_O      BIT(2)
#define  GPIO2_O      BIT(1)
#define  GPIO1_O      BIT(0)
#define GPIO_DATA_OUT           0x0C


#define GPIO_DATA_MASK          0x003F

/* Version */
#define VERSION_REG             0x2C
#define VERSION_MASK            0x1C00

/******* Test registers *******/
#define BER_REG                 0x30
#define BER_MASK                0x8000
#define PSM_REG                 0x31
#define PSM_MASK                0x0008
#define PAEN_REG                0x08
#define PAEN_MASK               0x8000


/*-------------------------------------------------------------------------
                    Register 24 Description
                     
*/
#define PLL_LOCK_IRQ                 BIT(15) // The Local Oscillator 1 Lock Detect Interrupt bit indicates whether the LO1 PLL is in or out of lock.
#define RAM_ADDR_ERR__TX_DONE_IRQ    BIT(14) // The Packet RAM Address Error Interrupt or Stream Mode TX Done Interrupt bit is a multiplexed interrupt status bit.
/*
1. Arbitration busy error - When in Packet Data Mode, a SPI access to Packet RAM was attempted during packet reception or transmission.
The arb_busy_mask controls the interrupt generation for this mode.
2. RX Done - When in Streaming Data Mode, the RX reception is complete and the transceiver has returned to Idle Mode.
The rx_done_mask controls interrupt generation in this mode.*/
#define ARB_BUSY_ERR__RX_DONE_IRQ    BIT(13)
#define STRM_DATA_ERR                BIT(12) // The Stream Mode Data Error Interrupt bit indicates Stream Mode data read or write error
/*
The ATTN signal has been asserted or the modem has reached a Power-up complete condition after a reset.*/
#define ATTN_IRQ                     BIT(10)
/*
While in Doze Mode, a tmr_cmp2 match has occurred and the MC1321x will return to Idle Mode.*/
#define DOZE_IRQ                     BIT(9)
#define TMR1_IRQ                     BIT(8)  // The Timer Compare 1 Interrupt bit.
#define RX_RCVD_IRQ__RX_STRM_IRQ     BIT(7)  // The RX Packet Received Interrupt or RX Stream Data Ready Interrupt bit is a multiplexed interrupt status bit
#define TX_SENT_IRQ__TX_STRM_IRQ     BIT(6)  // The TX Packet Sent Interrupt or TX Stream Data Needed Interrupt bit is a multiplexed interrupt status bit.
#define CCA_IRQ                      BIT(5)  // The Clear Channel Assessment Ready Interrupt bit
#define TMR3_IRQ                     BIT(4)  // The Timer Compare 3 Interrupt bit.
#define TMR4_IRQ                     BIT(3)  // The Timer Compare 4 Interrupt bit.
#define TMR2_IRQ                     BIT(2)  // The Timer Compare 2 Interrupt bit.
#define CCA                          BIT(1)  // The Clear Channel Assessment bit indicates channel busy or channel idle.
#define CRC_VALID                    BIT(0)  // The RX CRC Result bit denotes if the CRC is correct or not

#define IRQ_STATUS              0x24

#endif /* _MC13192_REGS_H */