#include "includes.h"

extern INT8U g_phy_mode;
TRxPacket *g_rx_pack_struct;
INT8U gu8IRQValue = 0;

static INT8U IRQPinLow(void);

/*-------------------------------------------------------------------------------------------------------------
    
-------------------------------------------------------------------------------------------------------------*/
void Phy_Init(INT8U clko_freq_n)
{
  Phy_RESET_to_low();
  Phy_Set_poll_mode();
  Phy_RESET_to_high();      //     
  while ( 1 )
  {
    if ( Phy_poll_interrupt() == TRUE )  //    
    {
      if ( (SPI_Read(IRQ_STATUS) & ATTN_IRQ)  != 0 ) break; //  ,     
    }
  }


  //  SPI_Write(0x11,0x80FF);   /* Eliminate Unlock Conditions due to L01 */

  SPI_Write(TMR_CMP1_A,TMR_CMP_DIS);      /* Disable TC1. */
  SPI_Write(TMR_CMP2_A,TMR_CMP_DIS);      /* Disable TC2. */
  SPI_Write(TMR_CMP3_A,TMR_CMP_DIS);      /* Disable TC3. */
  SPI_Write(TMR_CMP4_A,TMR_CMP_DIS);      /* Disable TC4. */
  SPI_Write(CONTROL_B, CT_BIAS_EN + MISO_HIZ_EN + BIT(10) + CLKO_DOZE_EN);  /* Enable CLKo in Doze */
  SPI_Write(GPIO_DATA_OUT,IRQB_DRV1 + IRQB_DRV0 + IRQB_PUP_EN + GPIO1_O);   //  IC  
  SPI_Write(CCA_THRESHOLD,0xA08D);   /* New cal value */
  SPI_Write(IRQ_MASK, ATTN_MASK + PLL_LOCK_MASK + ACOMA_EN + BIT(6) + DOZE_MASK + TMR1_MASK);   /* Acoma, TC1, Doze, ATTN masks, LO1, CRC */
  SPI_Write(CONTROL_A,  BIT(14) + CCA_MASK + TX_SENT_MASK + RX_RCVD_MASK + CCA_TYPE1);          /* CCA, TX, RX, energy detect */
  SPI_Write(GPIO_DIR, GPIO1234_DRV1 + GPIO1234_DRV0 + GPIO1_OEN + GPIO7_IEN + GPIO6_IEN + GPIO5_IEN + GPIO4_IEN + GPIO3_IEN + GPIO2_IEN);

  Phy_Set_CLKo_freq(clko_freq_n);


  /* Advance the state variable to Idle */
  g_phy_mode = IDLE_MODE;
}


/*-------------------------------------------------------------------------------------------------------------
 * IRQIsr : MC13192 initiated IRQ interrupt handler
 *
 * Parameters : None
 *
 * Return : The interrupt will RTI unless valid data is recvd.
 * In this case a PDDataIndication function call-back will be executed first,
 * followed by a RTI.
-------------------------------------------------------------------------------------------------------------*/
void interrupt IRQIsr(void)
{

  volatile INT16U status, rg;       /* Result of the status register read */
  INT8U dlen = 0;                        /* Data length for the RX packet */

  CLEAR_IRQ_FLAG;                        /* Acknowledge the interrupt. MC13192 IRQ pin still low. */
  status = SPI_Read(IRQ_STATUS);         /* Read the MC13192 status register.*/

  /* Test for IRQ glitch or possible fast double IRQ. */

  if ( IRQPinLow() )
  {
    /* IRQ is low. Re-read status. */
    CLEAR_IRQ_FLAG;                      /* Acknowledge the interrupt. MC13192 IRQ pin still low. */
    rg = SPI_Read(IRQ_STATUS);
    status = status | rg;                /* Actual status is the OR of both. */
  }

  status &= TX_SENT_IRQ__TX_STRM_IRQ | RX_RCVD_IRQ__RX_STRM_IRQ | ATTN_IRQ | CRC_VALID | TMR1_IRQ | CCA_IRQ | PLL_LOCK_IRQ | DOZE_IRQ;

  if ( (status & ~(CRC_VALID)) == 0 ) return; //        

  //    DOZE     2
  if ( (status & DOZE_IRQ) != 0 )
  {
    SPI_Write(TMR_CMP2_A, TMR_CMP_DIS);                     //  
    SPI_Write(CONTROL_B, SPI_Read(CONTROL_B) & (~DOZE_EN)); //    DOZE
    g_phy_mode = IDLE_MODE;
  }

  //   DOZE, HIBERNATE  OFF   ATTN  RESET
  if ( (status & ATTN_IRQ) != 0 )
  {
    status = SPI_Read(RST_IND) & RESET_IND;       /* Read the MC13192 reset indicator register.*/
    if ( status == 0 )
    {
      g_phy_mode = MC13192_RESET_MODE;    /* Set the rtx_state mirror to idle with attn.*/
      Phy_Indicate_Reset();
    }
    else g_phy_mode = IDLE_MODE_ATTN;       /* Set the rtx_state mirror to idle with attn. */
    return;
  }


  if ( (status & PLL_LOCK_IRQ) != 0 ) //   .       IDLE
  {
    Phy_RxTxEn_to_low();
    SPI_Write(CONTROL_A, (SPI_Read(CONTROL_A) & (~TMR_TRIG_EN) & (~XCVR_SEQ1) & (~XCVR_SEQ0)));  //     IDLE
    g_phy_mode = IDLE_MODE;
    return;
  }


  /* If rx is done */
  if ( (status & RX_RCVD_IRQ__RX_STRM_IRQ) != 0 )
  {

  }
}

/*-------------------------------------------------------------------------------------------------------------
 * IRQPinLow: Checks IRQ Pin to see if is low.
 * Return : 1 if IRQ is Low.
-------------------------------------------------------------------------------------------------------------*/
static INT8U IRQPinLow(void)
{

  asm  BIL irq
  return 0;
  asm irq:
  return 1;
}

/*-------------------------------------------------------------------------------------------------------------
 * Transmit data packet
 *
 * Parameters : *psPacket - packet pointer
 *
 * Return : Status
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Data_Send(tTxPacket *psPacket)
{
  if ( g_phy_mode == IDLE_MODE )
  {
    Phy_Write_SMAC_packet(psPacket);   /* Load the data into packet RAM */
    Phy_Set_Mode(TX_MODE);             /* transmit it */

    while ( g_phy_mode != IDLE_MODE )
    {
      MCU_LOW_POWER_WHILE();
    }
    return SUCCESS;
  }
  else
  {
    return RX_ON;
  }
}


/*-------------------------------------------------------------------------------------------------------------
   Hibernate the MC13192 (very low current, no CLKO).
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_to_Hibernate(void)
{
  INT16U val;

  g_phy_mode = HIBERNATE_MODE;
  val = SPI_Read(CONTROL_B);       /* Read MC13192 Hiberate register.*/
  val &= 0xFFFC;
  val |= 0x0002;                   /* Hiberate enable */
  SPI_Write(CONTROL_B, val);       /* Write back to MC13192 to enable hibernate mode.*/
  return SUCCESS;                  /* Call back to MAC layer indicating success. */
}


/*-------------------------------------------------------------------------------------------------------------
  Doze the MC13192 (Low current, CLKO <= 1MHz).
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_to_Doze(INT32U u32Timeout)
{
  INT16U val;
  INT32U u32CurrentTime;

  g_phy_mode = DOZE_MODE;
  val = SPI_Read(CONTROL_B);    /* Read MC13192 Doze register.*/
  if ( u32Timeout == 0 )        /* ACOMA mode, with clkout */
  {
    val = SPI_Read(IRQ_MASK);
    val &= 0xFE0D;   /* Change the IRQ_Mask to set Acoma en,*/
    val |= 0x0100;   /* doze irq disabled and tmr2 disabled*/
    SPI_Write(IRQ_MASK, val);

    val = SPI_Read(CONTROL_B);       /* Read MC13192 Doze register.*/
    val &= 0xFDFC;
    val |= 0x0201;                   /* Doze (acoma) & CLKOUT enable */
    SPI_Write(CONTROL_B, val);       /* Write back to MC13192 to enable hibernate mode.*/
  }
  else                               /* DOZE mode with timeout, no clkout */
  {
    val = SPI_Read(IRQ_MASK);
    val &= 0xFE0D;   /* Change the IRQ_Mask to set Acoma dis,*/
    val |= 0x0012;   /* doze enabled and tmr2 enabled*/
    SPI_Write(IRQ_MASK, val);

    u32CurrentTime = Phy_Read_Timestamp();
    u32Timeout += u32CurrentTime;
    SPI_Write( TMR_CMP2_A, ((INT16U)(u32Timeout>>16)&0x00FF) );
    SPI_Write( TMR_CMP2_B, ((INT16U)u32Timeout)&0xFFFF );  /* Enable the Timer 2 and save the timeout value*/
    val = SPI_Read(CONTROL_B);
    val &= 0xFDFC;            /* Disable CLKOinDozeand */
    val |= 0x0001;            /* enter in Doze mode */
    SPI_Write(CONTROL_B, val);
  }
  return SUCCESS;             /* Call back to MAC layer indicating success. */
}


/*-------------------------------------------------------------------------------------------------------------
   Wake the MC13192 from Hibernate or Doze.
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Wake_by_ATTN(void)
{
  INT16U val;

  _PTDD.Bits.PTDD0 = 0;       /* Assert ATTN */
  _PTDD.Bits.PTDD0 = 1;       /* Deassert ATTN */

  while ( g_phy_mode != IDLE_MODE_ATTN )
  {
    MCU_LOW_POWER_WHILE();                    /* Wait until ATTN */
  }
  val = SPI_Read(CONTROL_B);       /* Read MC13192 Hibernate/Doze register.*/
  val &= (~(HIB_EN)) & (~(DOZE_EN));  /* Hiberate and Doze disable */
  SPI_Write(CONTROL_B, val);       /* Write back to MC13192 to disable hibernate and doze mode.*/
  SPI_Write(TMR_CMP2_A, TMR_CMP_DIS);   /* Disable Timer2 To avoid a T2 int because of doze w/timeout*/
  g_phy_mode = IDLE_MODE;
  return SUCCESS;
}


/*-------------------------------------------------------------------------------------------------------------
 * Set the MC13192 receiver ON (with optional timeout)
 *
 * Parameters : *rx_packet, timeout - Packet pointer for received data and
 * timeout
 *
 * Return : Status
 *
 * Notes : Timeout of 0 disables the timeout.
 * The actual timeout period is the timeout value times the MC13192 timer rate from MLMESetMC13192TmrPrescale.
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Init_Reciving(TRxPacket *rxpack_struct, INT32U timeout)
{
  INT8U status = 0;

  g_rx_pack_struct = rxpack_struct;      /* Assign the rx_packet to SMAC global. */
  if ( timeout == 0 )
  {          /* Timeout disabled */
    /* Just enable the receiver */
    status = Phy_Set_Mode(RX_MODE);
  }
  else
  {
    /* Timeout requested. Get the current time and add the timeout value. */
    /* Erratum: In order to avoid the missing Irq, we reset the timer1 */
    Phy_Load_Event_Timer(0x00000000);
    status = Phy_LoadEnable_Tmr1_comp(timeout);        /* Set the timeout in TC1*/
    status = Phy_Set_Mode(RX_MODE_W_TIMEOUT);
  }
  return status;
}


/*-------------------------------------------------------------------------------------------------------------
 * Set the MC13192 operating channel.
 *
 * Parameters : ch - Channel number (0-15)
 *
 * Channel frequencies :
 *  0 : 2.405GHz
 *  1 : 2.410GHz
 *  2 : 2.415GHz
 *  3 : 2.420GHz
 *  4 : 2.425GHz
 *  5 : 2.430GHz
 *  6 : 2.435GHz
 *  7 : 2.440GHz
 *  8 : 2.445GHz
 *  9 : 2.450GHz
 *  10: 2.455GHz
 *  11: 2.460GHz
 *  12: 2.465GHz
 *  13: 2.470GHz
 *  14: 2.475GHz
 *  15: 2.480GHz
 *
 * Return : Status
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Set_Channel(INT8U u8Channel)
{
  switch ( u8Channel )
  {
    case 0x00:
      SPI_Write(LO1_IDIV_ADDR,0x0F95);
      SPI_Write(LO1_NUM_ADDR,0x5000);
      break;
    case 0x01:
      SPI_Write(LO1_IDIV_ADDR,0x0F95);
      SPI_Write(LO1_NUM_ADDR,0xA000);
      break;
    case 0x02:
      SPI_Write(LO1_IDIV_ADDR,0x0F95);
      SPI_Write(LO1_NUM_ADDR,0xF000);
      break;
    case 0x03:
      SPI_Write(LO1_IDIV_ADDR,0x0F96);
      SPI_Write(LO1_NUM_ADDR,0x4000);
      break;
    case 0x04:
      SPI_Write(LO1_IDIV_ADDR,0x0F96);
      SPI_Write(LO1_NUM_ADDR,0x9000);
      break;
    case 0x05:
      SPI_Write(LO1_IDIV_ADDR,0x0F96);
      SPI_Write(LO1_NUM_ADDR,0xE000);
      break;
    case 0x06:
      SPI_Write(LO1_IDIV_ADDR,0x0F97);
      SPI_Write(LO1_NUM_ADDR,0x3000);
      break;
    case 0x07:
      SPI_Write(LO1_IDIV_ADDR,0x0F97);
      SPI_Write(LO1_NUM_ADDR,0x8000);
      break;
    case 0x08:
      SPI_Write(LO1_IDIV_ADDR,0x0F97);
      SPI_Write(LO1_NUM_ADDR,0xD000);
      break;
    case 0x09:
      SPI_Write(LO1_IDIV_ADDR,0x0F98);
      SPI_Write(LO1_NUM_ADDR,0x2000);
      break;
    case 0x0A:
      SPI_Write(LO1_IDIV_ADDR,0x0F98);
      SPI_Write(LO1_NUM_ADDR,0x7000);
      break;
    case 0x0B:
      SPI_Write(LO1_IDIV_ADDR,0x0F98);
      SPI_Write(LO1_NUM_ADDR,0xC000);
      break;
    case 0x0C:
      SPI_Write(LO1_IDIV_ADDR,0x0F99);
      SPI_Write(LO1_NUM_ADDR,0x1000);
      break;
    case 0x0D:
      SPI_Write(LO1_IDIV_ADDR,0x0F99);
      SPI_Write(LO1_NUM_ADDR,0x6000);
      break;
    case 0x0E:
      SPI_Write(LO1_IDIV_ADDR,0x0F99);
      SPI_Write(LO1_NUM_ADDR,0xB000);
      break;
    case 0x0F:
      SPI_Write(LO1_IDIV_ADDR,0x0F9A);
      SPI_Write(LO1_NUM_ADDR,0x0000);
      break;
    default:
      SPI_Write(LO1_IDIV_ADDR,0x0F97);
      SPI_Write(LO1_NUM_ADDR,0xD000);
      return ERROR;
  }
  return SUCCESS;
}


/*-------------------------------------------------------------------------------------------------------------
 * Set the MC13192 transceive operation.
 *
 * Parameters : Operational mode
 *
 * Return : Status
-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Set_Mode(INT8U u8ReqMode)
{
  INT16U rg;

  Phy_RxTxEn_to_low();
  rg = SPI_Read(CONTROL_A) & (~BIT(3)) & (~XCVR_SEQ1) & (~XCVR_SEQ0);

  switch ( u8ReqMode )
  {
    case IDLE_MODE:                         /* Write Idle */
#if defined (LNA)
      MC13192_LNA_CTRL = LNA_OFF;
#endif
#if defined (PA)
      MC13192_PA_CTRL = PA_OFF;
#endif
      rg |= IDLE_MODE;
      g_phy_mode = IDLE_MODE;
      SPI_Write(CONTROL_A, rg);
      break;

    case RX_MODE:                           /* Write RX */
#if defined (LNA)
      MC13192_LNA_CTRL = LNA_ON;
#endif
#if defined (PA)
      MC13192_PA_CTRL = PA_OFF;
#endif
#if defined (ANTENNA_SWITCH)
      MC13192_ANT_CTRL2 = ANT_CTRL_ON;    /* Turn on the RX antenna */
      MC13192_ANT_CTRL = ANT_CTRL_OFF;    /* Turn off the TX antenna */
#endif
      g_phy_mode = RX_MODE;
      rg |= RX_MODE;
      SPI_Write(CONTROL_A, rg);
      Phy_RxTxEn_to_high();
      break;

    case TX_MODE:               /* Write Tx. Note: force LO lock not used */
#if defined (PA)
      MC13192_PA_CTRL = PA_ON;
#endif
#if defined (LNA)
      MC13192_LNA_CTRL = LNA_OFF;
#endif
#if defined (ANTENNA_SWITCH)
      MC13192_ANT_CTRL2 = ANT_CTRL_OFF;   /* Turn off the RX antenna */
      MC13192_ANT_CTRL = ANT_CTRL_ON;     /* Turn on the TX antenna */
#endif
      rg |= TX_MODE;
      g_phy_mode = TX_MODE;
      SPI_Write(CONTROL_A, rg);
      Phy_RxTxEn_to_high();
      break;

    case PULSE_TX_MODE: /* Write Tx. Note: force LO lock not used */

#if defined (PA)
      MC13192_PA_CTRL = PA_ON;
#endif

#if defined (LNA)
      MC13192_LNA_CTRL = LNA_OFF;
#endif

#if defined (ANTENNA_SWITCH)
      MC13192_ANT_CTRL2 = ANT_CTRL_OFF; /* Turn off the RX antenna */
      MC13192_ANT_CTRL = ANT_CTRL_ON; /* Turn on the TX antenna */
#endif

      g_phy_mode = PULSE_TX_MODE;
      rg |= TX_MODE;
      SPI_Write(CONTROL_A, rg);
      Phy_RxTxEn_to_high();
      break;

    default:
      return ERROR;
  }
  return SUCCESS;
}


/*
 * Measure channel energy.
 *
 * Parameters : None
 *
 * Return : Energy
 * Actual power returned is: -(power/2)
 * Global calibration required for accuracy (from MLMEMC13192FEGainAdjust).
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Get_EnergyLev (void)
/* Note: Actual power returned is: -(power/2) */
/* Global calibration required for accuracy. */
{
  INT16U rg;
  INT8U u8Power;

  g_phy_mode = CCA_MODE;                /* Write energy detect mode */
  rg = (SPI_Read(CONTROL_A) & 0xFFF8) | CCA_MODE;
  SPI_Write(CONTROL_A, rg);
  Phy_RxTxEn_to_high();
  while ( g_phy_mode != IDLE_MODE )       /* Wait for energy detect to complete */
  {
    MCU_LOW_POWER_WHILE();
  }
  rg = (SPI_Read(RX_STATUS) & 0xFF00);
  u8Power = (rg >> 8);
  return u8Power;
}


/*
 * Report energy from last successful RX packet.
 *
 * Parameters : None
 *
 * Return : Energy
 * Actual power returned is: -(power/2)
 * Global calibration required for accuracy (from MLMEMC13192FEGainAdjust).
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Read_LinkQuality (void)
/* Note: Actual power returned is: -(power/2) */
/* Global calibration required for accuracy. */
{
  INT16U rg;
  INT8U u8Power;

  rg = SPI_Read(RX_STATUS);
  u8Power = ((rg & 0xFF00) >> 8);
  return u8Power;
}


/*
 * Get MC13192 timer value.
 *
 * Parameters : None
 *
 * Return : Timer value
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT32U Phy_Read_Timestamp(void)
{
  INT32U u32UpperWord, u32LowerWord;
  INT32U u32CurrentTime;

  DISABLE_INTERRUPT;
  u32UpperWord = SPI_Read(TIMESTAMP_HI_ADDR);
  u32LowerWord = SPI_Read(TIMESTAMP_LO_ADDR);
  u32UpperWord &= TIMESTAMP_HI_MASK;              /* Clears TS_HELD bit. */
  u32CurrentTime = (INT32U) (u32UpperWord << 16) | u32LowerWord;
  ENABLE_INTERRUPT;
  return u32CurrentTime;
}


/*
 * Set MC13192 CLKo frequency.
 *
 * Parameters : freq - Frequency value enumeration (0-7)
 *
 *  Freq    Output Frequency
 *  0       16MHz (Recommended default)
 *  1       8MHz
 *  2       4MHz
 *  3       2MHz
 *  4       1MHz
 *  5       62.5kHz
 *  6       32.786kHz
 *  7       16.393kHz
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Set_CLKo_freq(INT8U u8Freq)
{
  volatile INT16U val;

  val = SPI_Read(CLKS_ADDR); /* Read register and re-write */
  val &= 0xFFF8;
  val |= u8Freq;
  SPI_Write(CLKS_ADDR, val);
  return SUCCESS;
}


/*
 * Set MC13192 timer frequency.
 *
 * Parameters : freq - Frequency value enumeration (0-7)
 *
 *  Freq    Output Frequency
 *  0       2MHz
 *  1       1MHz
 *  2       500kHz
 *  3       250kHz (Recommended default)
 *  4       125kHz
 *  5       62.5kHz
 *  6       31.25kHz
 *  7       15.625kHz
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Set_Event_Timer_Prescaler (INT8U freq)
{
  volatile INT16U val;

  val = SPI_Read(PRESCALE_ADDR);
  val &= 0xFFF8;
  val |= freq;
  SPI_Write(PRESCALE_ADDR, val);
  return SUCCESS;
}


/*
 * Set MC13192 timer value (i.e. initialize).
 *
 * Parameters : Timer value
 *
 * Return : None
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Load_Event_Timer(INT32U u32RequestedTime)
{
  INT16U reg;

  SPI_Write(TMR_CMP1_A, (INT16U) (u32RequestedTime >> 16) & 0x000000FF);
  SPI_Write(TMR_CMP1_B, (INT16U) u32RequestedTime & 0x0000FFFF);
  //       
  reg = SPI_Read(CONTROL_B);
  reg &= ~TMR_LOAD;
  SPI_Write(CONTROL_B, reg);
  reg |= TMR_LOAD;
  SPI_Write(CONTROL_B, reg);
  return;
}


/*
 * Set MC13192 timer compare value.
 *
 * Parameters : Timer value
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_LoadEnable_Tmr1_comp(INT32U u32CounterValue)
{
  /* Load the timeout value into T1 with Timer disabled. */
  SPI_Write(TMR_CMP1_A, (INT16U) ((u32CounterValue >> 16) & 0x000000FF)|0x000080FF);
  SPI_Write(TMR_CMP1_B, (INT16U) (u32CounterValue & 0x0000FFFF));
  /* Turn Timer1 mask on. */
  SPI_Write(TMR_CMP1_A, (INT16U) ((u32CounterValue >> 16) & 0x000000FF));
  SPI_Write(TMR_CMP1_B, (INT16U) (u32CounterValue & 0x0000FFFF));
  return SUCCESS;
}


/*
 * Disable MC13192 timer comparator TC1.
 *
 * Parameters : None
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Disable_Tmr1(void)
{
  SPI_Write(TMR_CMP1_A, TMR_CMP_DIS);
  SPI_Write(TMR_CMP1_B, 0x0000);
  return SUCCESS;
}


/*
 * Indicate a MC13192 reset condition.
 *
 * Parameters : None
 *
 * Return : None
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Indicate_Reset (void)
{
  //     RF 
}


/*
 * Force the MC13192 into a soft reset condition.
 *
 * Parameters : None
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Do_Soft_Reset(void)
{
  SPI_Write(RESET, 0x00);
  return SUCCESS;
}


/*
 * Adjust the MC13192s crystal trim value.
 *
 * Parameters : req_value - Trim value (0-255)
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Adjust_XTAL(INT8U u8TrimValue)
{
  INT16U rg;
  INT16U val;

  val = (u8TrimValue << 8);       /* Shift the req value into the higher half word*/
  rg = SPI_Read(XTAL_ADJ_ADDR);   /* Read the current value of XTAL Reg*/
  rg = ((rg & 0x00FF) | val);
  SPI_Write(XTAL_ADJ_ADDR, rg);
  return SUCCESS;
}


/*
 * Adjust the MC13192s gain compensator.
 *
 * Parameters : gain_value - Gain compensation value
 * (0 to 255. 128 is center point)
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Set_Power_Compensation(INT8U i8GainValue)
{
  INT16U rg;

  rg = SPI_Read(CCA_THRESH);       /* Read the current value of GAIN Reg*/
  rg = ((rg & 0xFF00) | i8GainValue);
  SPI_Write(CCA_THRESH, rg);
  return SUCCESS;
}


/*
 * Adjust the MC13192s Output power.
 *
 * Parameters : PA output adjust
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Adjust_Power_Level(INT8U val)
{
  INT16U rg;
  INT8U u8PAValue;

  switch ( val )
  {
    case MAX_POWER:         /* Sets the PA drive level and PA gain to MAX */
      u8PAValue = 0xFF;
      break;

    case MIN_POWER:
      u8PAValue = 0x00;   /* Sets the PA drive level and PA gain to MIN */
      break;

    default:
      if ( val > 15 )
      {
        return OVERFLOW;
      }
      else
      {
        u8PAValue = val;
      }
      break;
  }
  rg = SPI_Read(PA_LVL);    /* Read the current value of GAIN Register*/
  rg &= 0xFF00;
  if ( (val == MAX_POWER) || (val == MIN_POWER) )
  {
    rg |= u8PAValue;
  }
  else
  {
    rg |= ((u8PAValue << 4) | 0x000C);
  }
  SPI_Write(PA_LVL, rg);
  return SUCCESS;
}


/*
 * Returns the RFIC version number.
 *
 * Parameters : None
 *
 * Return : Version number
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT16U Phy_Read_RfICVersion(void)
{
  return(SPI_Read(VERSION_REG));
}

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_receive_packet(INT16U status)
{
  INT8U dlen;
  INT16U tmp;

  Phy_RxTxEn_to_low();
  if ( (status & CRC_VALID) == 0 )
  {
    //    CRC
    SPI_Write(CONTROL_A, (SPI_Read(CONTROL_A) & (~TMR_TRIG_EN)));   //  -  
    Phy_RxTxEn_to_high();  //  
    return FALSE;
  }
  else
  {

    dlen = (INT8U) (SPI_Read(RX_PKT_LEN) & 0x7F);
    if ( dlen < 5 ) /* Rx_pkt_length is bad when <5 because of CRC and byte codes. */
    {
      SPI_Write(CONTROL_A, (SPI_Read(CONTROL_A) & (~TMR_TRIG_EN)));   //  -  
      Phy_RxTxEn_to_high(); //  
      return FALSE;
    }

    tmp = SPI_ReadSecondWord(RX_PKT); /* Get the code bytes */
    if ( tmp != 0xFF7E )  //    SMAC
    {
      SPI_Write(CONTROL_A, (SPI_Read(CONTROL_A) & (~TMR_TRIG_EN)));   //  -  
      Phy_RxTxEn_to_high(); //  
      return FALSE;
    }


    g_phy_mode = IDLE_MODE;             /* Set the rtx_state to idle */
    Phy_Disable_Tmr1();
    g_rx_pack_struct->data_len = dlen;
    g_rx_pack_struct->stat = SUCCESS;
    Phy_Read_SMAC_packet(g_rx_pack_struct);         /* Read data from MC13192, check status */
    return TRUE;
  }
}

/*
 * Write a block of data to TX packet RAM, whichever is selected
 *
 * Parameters : length - Length of the block of data in bytes
 * *contents - Pointer to the data block
 *
 * Return : None
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Write_SMAC_packet(tTxPacket *psTxPkt)
{
  INT8U i, tmp, val;   /* Temporary counters */
  INT16U  rg;                         /* TX packet length register value */

  rg = SPI_Read(TX_PKT_LEN);    /* Read the TX packet length register contents*/
  rg = (0xFF80 & rg) | (psTxPkt->data_len + 4); /* Mask out old length setting and update. Add 2 for CRC and 2 for code bytes*/
  SPI_Write(TX_PKT_LEN, rg);    /* Update the TX packet length field */
  val = SPI1S;          /* Clear status register (possible SPRF, SPTEF)*/
  val = SPI1D;          /* Clear receive data register. SPI entirely ready for read or write*/
//  Phy_Disable_interrupts();   /* Necessary to prevent double SPI access */
  MODEM_CE_LOW;                 /* Enables MC13192 SPI */
  SPI1D = TX_PKT_RAM;           /* SPI TX ram data register */
  while ( !(SPI1S_SPRF) );        /* For this bit to be set, SPTED MUST be set. Now write content MSB*/
  val = SPI1D;          /* Clear receive data register. SPI entirely ready for read or write*/
  SPI1D = 0xFF;                 /* Send 1st Code Byte */
  while ( !(SPI1S_SPRF) );        /* For this bit to be set, SPTED MUST be set. Now write content MSB*/
  SPI1D = 0x7E;                 /* Send 2nd Code Byte */
  while ( !(SPI1S_SPRF) );        /* For this bit to be set, SPTED MUST be set. Now write content MSB*/
  val = SPI1D;          /* Clear receive data register. SPI entirely ready for read or write*/
  tmp = 0;               /* Byte counter for *contents */
  /* Word loop. Round up. */
  for ( i=0; i<((psTxPkt->data_len+1) >> 1); i++ )
  {
    SPI1D = psTxPkt->pdata[tmp];  /* Write MSB */
    while ( !(SPI1S_SPRF) );      /* For this bit to be set, SPTED MUST be set. Now write content LSB*/
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
    SPI1D = psTxPkt->pdata[tmp+1];   /* Write LSB */
    tmp = tmp + 2;            /* Increment byte counter */
    while ( !(SPI1S_SPRF) );      /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
  }
  MODEM_CE_HIGH;                /* Disables MC13192 SPI */
//  Phy_Restore_interrupts();   /* Restore MC13192 interrupt status */
}


/*
 * Read a block of data from RX packet RAM, whichever is selected
 *
 * Parameters : *length - Returned length of the block of data in bytes
 * *contents - Pointer to the data block storage
 *
 * Return : Status
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_Read_SMAC_packet(TRxPacket *psRxPkt)
{
  INT8U i, tmp;        /* Temporary counters. */
  volatile INT8U val;          /* Used by SPIClearRecieveDataReg to* flush SPI1D register during read*/
  INT8U  status = 0;        /* Holder for the return value */
  INT16U len;

  len = SPI_Read(RX_PKT_LEN) & 0x7F;   /* Read the RX packet length register contents*/

  /* MC13192 reports length with 2 CRC bytes and 2 Code bytes, remove them.*/
  if ( len >= 5 ) psRxPkt->data_len = len - 4; /* Less byte code and CRC*/
  else            psRxPkt->data_len = 0;


  if ( (psRxPkt->data_len >= 1) && (psRxPkt->data_len <= psRxPkt->max_dlen) )
  { /* If <5, the packet is garbage*/
    val = SPI1S;        /* Clear status register (SPRF, SPTEF) */
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
//    Phy_Disable_interrupts(); /* Necessary to prevent double SPI access */
    MODEM_CE_LOW;               /* Enables MC13192 SPI */
    SPI1D = RX_PKT | 0x80;      /* SPI RX ram data register */
    while ( !(SPI1S_SPRF) );     /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
    SPI1D = val;        /* Dummy write. Receive register of SPI will contain MSB garbage for first read*/
    while ( !(SPI1S_SPRF) );     /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
    SPI1D = val;        /* Dummy write. Receive register of SPI will contain LSB garbage for first read*/
    while ( !(SPI1S_SPRF) );     /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Clear receive data register. SPI entirely ready for read or write*/
    /* Byte codes */
    SPI1D = val;        /* Dummy write. Receive register of SPI will contain LSB garbage for first read*/
    while ( !(SPI1S_SPRF) );     /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Read first code byte */
    SPI1D = val;        /* Dummy write. Receive register of SPI will contain LSB garbage for first read*/
    while ( !(SPI1S_SPRF) );     /* For this bit to be set, SPTED MUST be set.*/
    val = SPI1D;        /* Read second code byte */
    tmp = 0;             /* Byte counter for contents */
    /* Word loop. Round up. Deduct CRC. */
    for ( i=0; i<((len-1)>>1); i++ )
    {
      SPI1D = val;      /* Dummy write. Receive register of SPI will contain MSB*/
      while ( !(SPI1S_SPRF) );   /* For this bit to be set, SPTED MUST be set. Get MSB*/
      /* For a trailing garbage byte, just read and discard */
      if ( (tmp + 3) == len )
      {
        val = SPI1D;    /* Discard */
      }
      else
      {
        psRxPkt->pdata[tmp] = SPI1D;   /* Read MSB */
      }
      SPI1D = val;      /* Dummy write. Receive register of SPI will contain LSB*/
      while ( !(SPI1S_SPRF) );   /* For this bit to be set, SPTED MUST be set. Get LSB*/
      psRxPkt->pdata[tmp+1] = SPI1D;   /* Read LSB */
      tmp = tmp + 2;        /* Increment byte counter */
    }
    MODEM_CE_HIGH;               /* Disables MC13192 SPI */
    psRxPkt->stat = SUCCESS;
//    Phy_Restore_interrupts();  /* Restore MC13192 interrupt status */
  }
  /* Check to see if a larger packet than desired is received. */
  if ( psRxPkt->data_len > psRxPkt->max_dlen )
  {
    psRxPkt->stat = OVERFLOW;
  }
  return status;
}

/*
 * MC13192DisableInterrupts : Disable MC13192 interrupts
 *
 * Parameters : None
 *
 * Return : None
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Disable_interrupts(void)
{
  gu8IRQValue = _IRQSC.Byte;
  _IRQSC.Bits.IRQIE = 0;
}


/*
 * MC13192RestoreInterrupts : Restore MC13192 interrupts to previous condition
 *
 * Parameters : None
 *
 * Return : None
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Restore_interrupts(void)
{
  _IRQSC.Byte = gu8IRQValue;   /* Restore the context of the IRQ register from global*/
}

/*
 * Deassert the MC13192 RTXEN pin (forces IC to idle)
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_RxTxEn_to_low(void)
{
  _PTDD.Bits.PTDD1 = 0;      /* Deassert RTXEN */
}


/*
 * Assert the MC13192 RTXEN pin (initiates programmed cycle)
 */
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_RxTxEn_to_high(void)
{
  _PTDD.Bits.PTDD1 = 1;      /* Assert RTXEN */
}

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_RESET_to_low(void)
{
  _PTDD.Bits.PTDD3 = 0;
}
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_RESET_to_high(void)
{
  _PTDD.Bits.PTDD3 = 1;
}


/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Set_poll_mode(void)
{
  _IRQSC.Bits.IRQMOD = 1;   //     
  _IRQSC.Bits.IRQIE  = 0;   //   ,   
  _IRQSC.Bits.IRQEDG = 0;   //   
  _IRQSC.Bits.IRQACK = 1;   //     IRQ
  _IRQSC.Bits.IRQPE  = 1;   //   IRQ
}

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Set_int_mode(void)
{
  _IRQSC.Bits.IRQMOD = 0;   //   
  _IRQSC.Bits.IRQIE  = 1;   //    
  _IRQSC.Bits.IRQEDG = 0;   //   
  _IRQSC.Bits.IRQACK = 1;   //     IRQ
  _IRQSC.Bits.IRQPE  = 1;   //   IRQ
}


/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
INT8U Phy_poll_interrupt(void)
{
  if (_IRQSC.Bits.IRQF!=0)
  {
    _IRQSC.Bits.IRQACK = 1; //   
    return(TRUE);
  }
  return(FALSE);
}

/*
*    Loads the transmit RAM with a PRBS9 data pattern.
*
*    Parameters: Transmit buffer pointer
*
*    Return:     none.
*/
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Load_TX_w_PRBS9_pattern (tTxPacket *psPacket)
{
  INT8U u8c1;                           /* Byte counter */
  INT8U u8c2;                           /* Bit counter */
  INT16U u16t1;                         /* LFSR */
  INT16U u16t2;                         /* LFSR output */
  INT16U u16t3;                         /* LFSR feedback tap */
  INT8U u8t4;                           /* Assembled transmit byte */

  u16t1 = 0x01FF;                     /* Initialize the LFSR */
  for ( u8c1=0; u8c1<64; u8c1++ )       /* Byte counter */
  {
    u8t4 = 0x00;                    /* Initialize the byte */
    for ( u8c2=0; u8c2<8; u8c2++ )        /* Bit counter */
    {
      u16t2 = (u16t1 & 0x0001);       /* LFSR output */
      if ( u16t2 == 0x0001 )
      {
        u8t4 = u8t4 | 0x80;         /* Set/Clear byte based on LFSR output */
      }
      if ( u8c2 != 7 )
      {
        u8t4 = u8t4 >> 1;           /* LSBit will be first bit out of LFSR */
      }
      u16t3 = ((u16t1 & 0x0010) >> 4);/* LFSR tap */
      u16t1 = u16t1 >> 1;             /* Now shift the LFSR */
      if ( u16t2 == u16t3 )             /* Set/Clr the LFSR MSBit */
      {
        u16t1 = u16t1 & 0xFEFF;
      }
      else
      {
        u16t1 = u16t1 | 0x0100;
      }
    }
    psPacket->pdata[u8c1] = u8t4;
  }
  psPacket->data_len = 64;
  Phy_Write_SMAC_packet(psPacket);            /* Load the data into packet RAM */
}


/*
*    Function:     Execute a test mode
*    Parameters: packet pointer, desired test mode
*    Return:        none
*/
/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void Phy_Do_Test (tTxPacket *psPacket, INT8U u8mode)
{
  INT16U u16reg;
  INT32U i;
  switch ( u8mode )
  {
    /* Continuously transmit a PRBS9 pattern. */
    case PULSE_TX_PRBS9:
      Phy_Load_TX_w_PRBS9_pattern (psPacket);   /* Load the TX RAM */
      Phy_Set_Mode(PULSE_TX_MODE); /* transmit it. Interrupt
                                              * routine will retransmit
                                              * after completion
                                              */
      break;
      /* Sets the device back to original IDLE mode. */
    case FORCE_IDLE:

      Phy_Disable_interrupts();  /* Mask off interrupts from MC13192 */
      Phy_RxTxEn_to_low();             /* Force MC13192 to IDLE */
      g_phy_mode = IDLE_MODE;      /* set the SW mode to IDLE */
      Phy_Restore_interrupts();

      for ( i=0; i < 200; i++ );
      Phy_Set_Mode(IDLE_MODE);  /* Set to IDLE */
      u16reg = SPI_Read(BER_REG);       /* Turn off BER mode */
      u16reg = (u16reg & ~(BER_MASK));
      SPI_Write(BER_REG,u16reg);
      u16reg = SPI_Read(PSM_REG);       /* Turn off PSM test mode */
      u16reg = (u16reg & ~(PSM_MASK));
      SPI_Write(PSM_REG,u16reg);

      u16reg = SPI_Read(PAEN_REG);      /* dis-enable the PA */
      u16reg = (u16reg & ~PAEN_MASK);
      SPI_Write(PAEN_REG,u16reg);

      Phy_Set_Mode(TX_MODE);    /* Realign TX */

      u16reg = SPI_Read(PAEN_REG);      /* enable the PA */
      u16reg = (u16reg | PAEN_MASK);
      SPI_Write(PAEN_REG,u16reg);

      break;
      /* Sets the device into continuous RX mode */
    case CONTINUOUS_RX:
      u16reg = SPI_Read(BER_REG);       /* Turn on BER mode */
      u16reg = (u16reg | BER_MASK);
      SPI_Write(BER_REG,u16reg);
      Phy_Set_Mode(RX_MODE);    /* Turn the receiver on */
      break;
      /* Sets the device to continuously transmit a 10101010 pattern */
    case CONTINUOUS_TX_MOD:
      SPI_Write(TX_PKT_RAM,0xAAAA);         /* Load the test pattern */
      u16reg = SPI_Read(PAEN_REG);      /* Disable the PA */
      u16reg = (u16reg & ~(PAEN_MASK));
      SPI_Write(PAEN_REG,u16reg);
      u16reg = SPI_Read(TX_PKT_LEN);    /* Set the length field */
      u16reg = ((u16reg & ~(TX_PKT_LEN_MASK)) | 0x0004);
      SPI_Write(TX_PKT_LEN,u16reg);
      Phy_Set_Mode(TX_MODE);    /* Do a transmit to initialize */
      while ( g_phy_mode != IDLE_MODE );    /* Wait till done */
      u16reg = SPI_Read(PAEN_REG);      /* Re-enable the PA */
      u16reg = (u16reg | PAEN_MASK);
      SPI_Write(PAEN_REG,u16reg);
      u16reg = SPI_Read(BER_REG);       /* Turn on BER mode */
      u16reg = (u16reg | BER_MASK);
      SPI_Write(BER_REG,u16reg);
      Phy_Set_Mode(TX_MODE);    /* Turn the transmitter on */
      break;
      /* Sets the device to continuously transmit an unmodulated CW */
    case CONTINUOUS_TX_NOMOD:
      u16reg = SPI_Read(PAEN_REG);      /* Disable the PA */
      u16reg = (u16reg & ~(PAEN_MASK));
      SPI_Write(PAEN_REG,u16reg);
      u16reg = SPI_Read(TX_PKT_LEN);    /* Set the length field */
      u16reg = ((u16reg & ~(TX_PKT_LEN_MASK)) | 0x0004);
      SPI_Write(TX_PKT_LEN,u16reg);
      Phy_Set_Mode(TX_MODE);    /* Do a transmit to initialize */
      while ( g_phy_mode != IDLE_MODE );    /* Wait till done */
      u16reg = SPI_Read(PAEN_REG);      /* Re-enable the PA */
      u16reg = (u16reg | PAEN_MASK);
      SPI_Write(PAEN_REG,u16reg);
      u16reg = SPI_Read(BER_REG);       /* Turn on BER mode */
      u16reg = (u16reg | BER_MASK);
      SPI_Write(BER_REG,u16reg);
      u16reg = SPI_Read(PSM_REG);       /* Turn off PSM */
      u16reg = (u16reg | PSM_MASK);
      SPI_Write(PSM_REG,u16reg);
      Phy_Set_Mode(TX_MODE);    /* Turn the transmitter on */
      break;
  }
}
