unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ActnList, Menus, Placemnt, OoMisc, AdPort, ComCtrls,
  ExtCtrls, StdCtrls, TimerLst, TeEngine, Series, TeeProcs, Chart, Buttons;

const
   ENQ:byte = 5;
   ACK:byte = 6;
   DC1:byte = $11;
   DC2:byte = $12;
   STREAM_ON:byte = $13;
   STREAM_OFF:byte = $14;


type
  TProtState = (psIdle, psInit, psReadAns);

  TPacket = record
    pos : integer;
    buf : array[0..255] of byte;
  end;



  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    ActionList1: TActionList;
    actSetComPortOpt: TAction;
    actTestTerminal: TAction;
    FirstCOMportsettings1: TMenuItem;
    estinisterminalas1: TMenuItem;
    ImageList1: TImageList;
    ApdComPort1: TApdComPort;
    FormStorage1: TFormStorage;
    StatusBar: TStatusBar;
    Timer1: TTimer;
    Panel1: TPanel;
    Panel2: TPanel;
    RxTimerList1: TRxTimerList;
    tmrPacketTimeout: TRxTimerEvent;
    tmrDataTimeout: TRxTimerEvent;
    Chart1: TChart;
    Series1: TFastLineSeries;
    Label1: TLabel;
    lblCh: TLabel;
    BitBtn1: TBitBtn;
    btnStartStop: TBitBtn;
    procedure actSetComPortOptExecute(Sender: TObject);
    procedure actTestTerminalExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormStorage1RestorePlacement(Sender: TObject);
    procedure tmrPacketTimeoutTimer(Sender: TObject);
    procedure tmrDataTimeoutTimer(Sender: TObject);
    procedure ApdComPort1TriggerAvail(CP: TObject; Count: Word);
    procedure BitBtn1Click(Sender: TObject);
    procedure btnStartStopClick(Sender: TObject);
  private
    ffirstactivate: boolean;
    protstate  : TProtState;    //   
    cmd_sym    : byte;
    packet     : TPacket;
    procedure ActivateAll(Sender: TObject);
    procedure ActivateComPort;
    procedure ResetPacket;
    procedure SendPacket(const block; const size: word; const timeout: integer);
  public
    procedure UpdateStatusBar;
  end;

var
  frmMain: TfrmMain;

implementation

uses ComPortSettings, Terminal;

{$R *.dfm}
procedure TfrmMain.FormCreate(Sender: TObject);
var inifname:string;
begin
  ffirstactivate  := true;
  inifname := ExtractFileName(Application.ExeName);
  inifname := ChangeFileExt(inifname, '.ini');
  FormStorage1.IniFileName:= ExtractFilePath(Application.ExeName)+ inifname;
end;

procedure TfrmMain.FormStorage1RestorePlacement(Sender: TObject);
begin
  if ffirstactivate = false then exit;  //   RestorePlacement      
  ffirstactivate := false;

  Timer1.OnTimer := frmMain.ActivateAll;  // OnShow     -     
  Timer1.Enabled         := true;
end;

procedure TfrmMain.ActivateAll(Sender: TObject);
begin
  frmMain.Timer1.OnTimer := Nil;
  Timer1.Enabled         := false;
  ActivateComPort;
  UpdateStatusBar;
end;

procedure TfrmMain.ActivateComPort;
var mstr:string;
begin
  try
    ApdComPort1.Open:=true;
    if (ApdComPort1.Open = false) then Abort;
  except
    mstr := 'It was not possible to open port Com'+IntToStr(ApdComPort1.ComNumber)+ '. Do you want to change settings? ';
    if MessageDlg(mstr, mtError	, [mbYes, mbNo], 0) = mrYes then
    begin
      actSetComPortOptExecute(Self);
    end;
  end
end;



procedure TfrmMain.actSetComPortOptExecute(Sender: TObject);
begin
  try
    with  TfrmComPortSettings.Create(Self) do
    begin
      ComPort:= ApdComPort1;
      try
        ShowModal;
      finally
        Free;
      end;
    end;
  finally
    UpdateStatusBar;
  end;
end;

procedure TfrmMain.actTestTerminalExecute(Sender: TObject);
begin
  try
    Self.Visible :=false;
    with  TfrmTestTerminal.Create(Self) do
    begin
      ApdComPort1.OnTriggerAvail := Nil;
      StatusBar.Panels[0].text:=Self.StatusBar.Panels[0].text;
      ShowModal;
      Free;
    end;
  finally
    Self.Visible :=true;
    ApdComPort1.OnTriggerAvail := ApdComPort1TriggerAvail;

  end;
end;


procedure TfrmMain.UpdateStatusBar;

  function port_state_str(port: TApdComPort):string;
  var str,s :string;
  begin
    str:='Com'+IntToStr(port.ComNumber);
    str:=str+' '+IntToStr(port.Baud);
    str:=str+' '+IntToStr(port.DataBits);
    case  port.Parity of
    pNone:  s:='N';
    pOdd:   s:='O';
    pEven:  s:='E';
    pMark:  s:='M';
    pSpace: s:='S';
    end;
    str:=str+' '+s;
    str:=str+' '+IntToStr(port.StopBits);

    if port.Open then
      str:=str+'. Port open'
    else
      str:=str+'. Port close';
    port_state_str := str;
  end;

begin
  with StatusBar do
  begin
    panels[0].text:=port_state_str(ApdComPort1)
  end;
end;




procedure TfrmMain.ResetPacket;
begin
  packet.pos := 0;
  tmrDataTimeout.Enabled   := False;
  tmrPacketTimeout.Enabled := True;
end;

procedure TfrmMain.SendPacket(const block; const size: word; const timeout: integer);
type
  TBytes = array[0..MaxInt - 1] of Byte;
var
    i   : integer;
    buf : array[0..63] of byte;
begin

  for i:=0 to size-1 do
  begin
    buf[i] := TBytes(block)[i];
  end;
  ApdComPort1.PutBlock(buf[0],size);
  tmrPacketTimeout.Interval := timeout;
  tmrPacketTimeout.Enabled  := True;
  tmrDataTimeout.Enabled    := False;
end;


procedure TfrmMain.tmrPacketTimeoutTimer(Sender: TObject);
begin
  ShowMessage('Not answer!');
end;

procedure TfrmMain.tmrDataTimeoutTimer(Sender: TObject);
var
  st: string;
  i : integer;
begin
  st :='';
  if  (protstate=psReadAns) and (Packet.pos>0) then
  begin
    for i:=0 to Packet.pos-1 do
    begin
      st := st + format('%.2X',[Packet.buf[i]]);
    end;
  end;
  tmrPacketTimeout.Enabled := True;
end;

procedure TfrmMain.ApdComPort1TriggerAvail(CP: TObject; Count: Word);
var
    i: integer;
    b: Byte;
begin
  //     
  tmrPacketTimeout.Enabled := False;  //    
  //tmrDataTimeout.Enabled   := True;
  //   
  for i:=1 to Count do
  begin
    b := Byte(ApdComPort1.GetChar);
    if (b and $80) <>0 then
    begin
      lblCh.Caption := IntToStr(b and $7F);
    end
    else
    begin
      Series1.AddY(b);
    end;


  end;



end;





procedure TfrmMain.BitBtn1Click(Sender: TObject);
begin
  Series1.Clear;
end;

procedure TfrmMain.btnStartStopClick(Sender: TObject);
begin
  if ApdComPort1.Open then
  begin
    ApdComPort1.Open :=false;
    btnStartStop.Caption := 'Start';
  end
  else
  begin
    ApdComPort1.Open := true;
    btnStartStop.Caption := 'Stop';
  end;
end;

end.
